import uwsgi

# send a raw signal to register with file_monitor subsystem
#uwsgi.signal(10, "/tmp/topolino")
uwsgi.signal(10, "/tmp")
#uwsgi.signal(10, "/root")

# send a raw signal to register with timer subsystem
uwsgi.signal(11, "3")
uwsgi.signal(11, "4")
uwsgi.signal(11, "8")


def application(e, s):
        s('200 Ok', [('Content-Type', 'text/html')])
        return "<h1>Hello World</h1>"



import uwsgi

def app1(e, sr):
    print(e)
    sr('200 Ok', [('Content-Type','text/plain')])
    return b'i Am a Python 3.x bytestring'

def app2(e, sr):
    print(e)
    sr('404 Not Found', [('Content-Type','text/plain')])
    return b'i Am a Python 3.x Not Found page'

uwsgi.applications = {b'': app1, b'/test': app2}



import os
NAME='router_spnego'

CFLAGS = os.popen('krb5-config --cflags gssapi').read().rstrip().split()
LDFLAGS = []
LIBS = os.popen('krb5-config --libs gssapi').read().rstrip().split()

GCC_LIST = ['router_spnego']



import os
NAME='v8'

CFLAGS = ['-Wno-deprecated-declarations']
LDFLAGS = []
LIBS = ['-lstdc++', '-lv8']
engine = os.environ.get('UWSGICONFIG_V8_ENGINE', '')
if engine == 'teajs':
    CFLAGS.append('-DUWSGI_V8_TEAJS -fexceptions')
    LIBS.append('-lteajs')
GCC_LIST = ['plugin', 'v8_uwsgi.cc', 'v8_commonjs.cc', 'v8_jsgi.cc']



import signal, sys
import time
import uwsgi
import os

print(uwsgi.opt)

sig_timeout = uwsgi.opt.get('test_mule_timeout', 10)
sig_to_send = uwsgi.opt.get('test_signal', signal.SIGINT)

def sig_handler(signum, frame):
    print('Hello from signal', signum)
    time.sleep(int(sig_timeout))
    sys.exit(0)

signal.signal(signal.SIGINT, sig_handler)
signal.signal(signal.SIGHUP, sig_handler)

time.sleep(1)

os.kill(uwsgi.masterpid(), int(sig_to_send))

while True:
    uwsgi.farm_get_msg()



import os

NAME = 'webdav'

CFLAGS = ['-Wno-deprecated-declarations']
CFLAGS += os.popen('xml2-config --cflags').read().rstrip().split()
LDFLAGS = []
LIBS = os.popen('xml2-config --libs').read().rstrip().split()

GCC_LIST = ['webdav']



import os
import sys
import uwsgiconfig as uc
import shutil

from setuptools import setup
from setuptools.dist import Distribution
from setuptools.command.install import install
from setuptools.command.install_lib import install_lib
from setuptools.command.build_ext import build_ext

try:
    from wheel.bdist_wheel import bdist_wheel
    HAS_WHEEL = True
except ImportError:
    HAS_WHEEL = False

"""
This is a hack allowing you installing
uWSGI and uwsgidecorators via pip and easy_install
since 1.9.11 it automatically detects pypy
"""

uwsgi_compiled = False


def get_profile():
    is_pypy = False
    try:
        import __pypy__
        is_pypy = True
    except ImportError:
        pass
    if is_pypy:
        profile = os.environ.get('UWSGI_PROFILE', 'buildconf/pypy.ini')
    else:
        profile = os.environ.get('UWSGI_PROFILE', 'buildconf/default.ini')
    if not profile.endswith('.ini'):
        profile = "%s.ini" % profile
    if not '/' in profile:
        profile = "buildconf/%s" % profile

    return profile


def patch_bin_path(cmd, conf):

    bin_name = conf.get('bin_name')

    if not os.path.isabs(bin_name):
        print('Patching "bin_name" to properly install_scripts dir')
        try:
            if not os.path.exists(cmd.install_scripts):
                os.makedirs(cmd.install_scripts)
            conf.set('bin_name',
                     os.path.join(cmd.install_scripts, conf.get('bin_name')))
        except:
            conf.set('bin_name', sys.prefix + '/bin/' + bin_name)


class uWSGIBuilder(build_ext):

    def run(self):
        global uwsgi_compiled
        if not uwsgi_compiled:
            conf = uc.uConf(get_profile())
            patch_bin_path(self, conf)
            uc.build_uwsgi(conf)
            uwsgi_compiled = True


class uWSGIInstall(install):

    def run(self):
        global uwsgi_compiled
        if not uwsgi_compiled:
            conf = uc.uConf(get_profile())
            patch_bin_path(self, conf)
            uc.build_uwsgi(conf)
            uwsgi_compiled = True
        install.run(self)


class uWSGIInstallLib(install_lib):

    def run(self):
        global uwsgi_compiled
        if not uwsgi_compiled:
            conf = uc.uConf(get_profile())
            patch_bin_path(self, conf)
            uc.build_uwsgi(conf)
            uwsgi_compiled = True
        install_lib.run(self)


if HAS_WHEEL:
    class uWSGIWheel(bdist_wheel):
        def finalize_options(self):
            bdist_wheel.finalize_options(self)
            self.root_is_pure = False


class uWSGIDistribution(Distribution):

    def __init__(self, *attrs):
        Distribution.__init__(self, *attrs)
        self.cmdclass['install'] = uWSGIInstall
        self.cmdclass['install_lib'] = uWSGIInstallLib
        self.cmdclass['build_ext'] = uWSGIBuilder
        if HAS_WHEEL:
            self.cmdclass['bdist_wheel'] = uWSGIWheel

    def is_pure(self):
        return False


setup(
    name='uWSGI',
    version=uc.uwsgi_version,
    description='The uWSGI server',
    author='Unbit',
    author_email='info@unbit.it',
    license='GPLv2+',
    descriptions='The uWSGI Platform',
    url='https://uwsgi-docs.readthedocs.io/en/latest/',
    py_modules=['uwsgidecorators'],
    distclass=uWSGIDistribution,
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'License :: OSI Approved :: GNU General Public License v2 or later (GPLv2+)',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3.12',
    ],
)



import os,sys

NAME='psgi'
CFLAGS = os.popen('perl -MExtUtils::Embed -e ccopts').read().rstrip().split()
LDFLAGS = os.popen('perl -MExtUtils::Embed -e ldopts').read().rstrip().split()
LIBS = []
for lib in LDFLAGS:
    if lib.startswith('-l'):
        LIBS.append(lib)

GCC_LIST = ['uwsgi_plmodule', 'psgi_loader', 'psgi_response', 'psgi_plugin']

for item in LDFLAGS:
    if item.endswith('DynaLoader.a'):
        GCC_LIST.append(item)



from uwsgidecorators import *
import Queue
from threading import Thread

queues = {}

class queueconsumer(object):

    def __init__(self, name, num=1, **kwargs):
        self.name = name
	self.num = num
        self.queue = Queue.Queue()
	self.threads = []
	self.func = None
	queues[self.name] = self


    @staticmethod
    def consumer(self):
        while True:
	    req = self.queue.get()
            print req
	    self.func(req)
	    self.queue.task_done()

    def __call__(self, f):
        self.func = f
	for i in range(self.num):
	    t = Thread(target=self.consumer,args=(self,))	
	    self.threads.append(t)
	    t.daemon = True
	    t.start()

@spool
def spooler_enqueuer(arguments):
    if 'queue' in arguments:
        queue = arguments['queue']
        queues[queue].queue.put(arguments)
    else:
        raise Exception("You have to specify a queue name")


def enqueue(*args, **kwargs):
    return spooler_enqueuer.spool(*args, **kwargs)



import os
import unittest
import time
from subprocess import Popen, PIPE


def touch(fname):
    with open(fname, 'a'):
        os.utime(fname, None)
    return


def pgrep_mules():
    cmd = ["pgrep", "-f", "uWSGI mule"]
    proc = Popen(cmd, stdout=PIPE)
    rv = set(map(int, proc.stdout.readlines()))
    proc.terminate()
    return rv


class MuleReloadingTest(unittest.TestCase):

    def test_reloading(self):
        try:
            uwsgi = Popen(["./uwsgi", "--master", "--mule", "--touch-mules-reload=/tmp/reload", "--socket=:0", "--auto-procname"])
            subprocesses = pgrep_mules()
            touch("/tmp/reload")
            time.sleep(1)
            new_subprocesses = pgrep_mules()
            proc_intersect = subprocesses.intersection(new_subprocesses)
            self.assertFalse(proc_intersect)
        finally:
            uwsgi.terminate()


if __name__ == "__main__":
    unittest.main()
